#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Pixmap Dial, a custom Qt widget
# Copyright (C) 2011-2022 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ---------------------------------------------------------------------------------------------------------------------
# Imports (Global)

from math import cos, floor, pi, sin

from PyQt5.QtCore import pyqtSlot, Qt, QEvent, QPointF, QRectF, QTimer, QSize
from PyQt5.QtGui import QColor, QConicalGradient, QFontMetrics, QPainterPath, QPen, QPixmap
from PyQt5.QtWidgets import QDial

from .commondial import CommonDial
from carla_shared import fontMetricsHorizontalAdvance

# ---------------------------------------------------------------------------------------------------------------------
# Widget Class

class PixmapDial(CommonDial):
    def __init__(self, parent, index=0):
        CommonDial.__init__(self, parent, index)

        self.fPixmap    = QPixmap(":/bitmaps/dial_01d.png")
        self.fPixmapNum = "01"

        if self.fPixmap.width() > self.fPixmap.height():
            self.fPixmapOrientation = self.HORIZONTAL
        else:
            self.fPixmapOrientation = self.VERTICAL

        self.updateSizes()

    def getBaseSize(self):
        return self.fPixmapBaseSize

    def updateSizes(self):
        self.fPixmapWidth  = self.fPixmap.width()
        self.fPixmapHeight = self.fPixmap.height()

        if self.fPixmapWidth < 1:
            self.fPixmapWidth = 1

        if self.fPixmapHeight < 1:
            self.fPixmapHeight = 1

        if self.fPixmapOrientation == self.HORIZONTAL:
            self.fPixmapBaseSize    = self.fPixmapHeight
            self.fPixmapLayersCount = self.fPixmapWidth / self.fPixmapHeight
        else:
            self.fPixmapBaseSize    = self.fPixmapWidth
            self.fPixmapLayersCount = self.fPixmapHeight / self.fPixmapWidth

        self.setMinimumSize(self.fPixmapBaseSize, self.fPixmapBaseSize + self.fLabelHeight + 5)
        self.setMaximumSize(self.fPixmapBaseSize, self.fPixmapBaseSize + self.fLabelHeight + 5)

        if not self.fLabel:
            self.fLabelHeight = 0
            self.fLabelWidth  = 0
            return

        metrics = QFontMetrics(self.fLabelFont)
        self.fLabelWidth  = fontMetricsHorizontalAdvance(metrics, self.fLabel)
        self.fLabelHeight = metrics.height()

        self.fLabelPos.setX(float(self.fPixmapBaseSize)/2.0 - float(self.fLabelWidth)/2.0)

        if self.fPixmapNum in ("01", "02", "07", "08", "09", "10"):
            self.fLabelPos.setY(self.fPixmapBaseSize + self.fLabelHeight)
        elif self.fPixmapNum in ("11",):
            self.fLabelPos.setY(self.fPixmapBaseSize + self.fLabelHeight*2/3)
        else:
            self.fLabelPos.setY(self.fPixmapBaseSize + self.fLabelHeight/2)

        self.fLabelGradient.setStart(0, float(self.fPixmapBaseSize)/2.0)
        self.fLabelGradient.setFinalStop(0, self.fPixmapBaseSize + self.fLabelHeight + 5)

        self.fLabelGradientRect = QRectF(float(self.fPixmapBaseSize)/8.0, float(self.fPixmapBaseSize)/2.0, float(self.fPixmapBaseSize*3)/4.0, self.fPixmapBaseSize+self.fLabelHeight+5)

    def setPixmap(self, pixmapId):
        self.fPixmapNum = "%02i" % pixmapId
        self.fPixmap.load(":/bitmaps/dial_%s%s.png" % (self.fPixmapNum, "" if self.isEnabled() else "d"))

        if self.fPixmap.width() > self.fPixmap.height():
            self.fPixmapOrientation = self.HORIZONTAL
        else:
            self.fPixmapOrientation = self.VERTICAL

        # special pixmaps
        if self.fCustomPaintMode == self.CUSTOM_PAINT_MODE_NULL:
            # reserved for carla-wet, carla-vol, carla-pan and color
            if self.fPixmapNum == "03":
                self.fCustomPaintMode = self.CUSTOM_PAINT_MODE_COLOR

            # reserved for carla-L and carla-R
            elif self.fPixmapNum == "04":
                self.fCustomPaintMode = self.CUSTOM_PAINT_MODE_CARLA_L

            # reserved for zita
            elif self.fPixmapNum == "06":
                self.fCustomPaintMode = self.CUSTOM_PAINT_MODE_ZITA

        self.updateSizes()
        self.update()

    @pyqtSlot()
    def slot_updatePixmap(self):
        self.setPixmap(int(self.fPixmapNum))

    def minimumSizeHint(self):
        return QSize(self.fPixmapBaseSize, self.fPixmapBaseSize)

    def sizeHint(self):
        return QSize(self.fPixmapBaseSize, self.fPixmapBaseSize)

    def changeEvent(self, event):
        CommonDial.changeEvent(self, event)

        # Force pixmap update if enabled state changes
        if event.type() == QEvent.EnabledChange:
            self.setPixmap(int(self.fPixmapNum))

    def paintDial(self, painter):
        if self.isEnabled():
            normValue = float(self.fRealValue - self.fMinimum) / float(self.fMaximum - self.fMinimum)
            target    = QRectF(0.0, 0.0, self.fPixmapBaseSize, self.fPixmapBaseSize)

            curLayer = int((self.fPixmapLayersCount - 1) * normValue)

            if self.fPixmapOrientation == self.HORIZONTAL:
                xpos = self.fPixmapBaseSize * curLayer
                ypos = 0.0
            else:
                xpos = 0.0
                ypos = self.fPixmapBaseSize * curLayer

            source = QRectF(xpos, ypos, self.fPixmapBaseSize, self.fPixmapBaseSize)
            painter.drawPixmap(target, self.fPixmap, source)

            # Custom knobs (Dry/Wet and Volume)
            if self.fCustomPaintMode in (self.CUSTOM_PAINT_MODE_CARLA_WET, self.CUSTOM_PAINT_MODE_CARLA_VOL):
                # knob color
                colorGreen = QColor(0x5D, 0xE7, 0x3D).lighter(100 + self.fHoverStep*6)
                colorBlue  = QColor(0x3E, 0xB8, 0xBE).lighter(100 + self.fHoverStep*6)

                # draw small circle
                ballRect = QRectF(8.0, 8.0, 15.0, 15.0)
                ballPath = QPainterPath()
                ballPath.addEllipse(ballRect)
                #painter.drawRect(ballRect)
                tmpValue  = (0.375 + 0.75*normValue)
                ballValue = tmpValue - floor(tmpValue)
                ballPoint = ballPath.pointAtPercent(ballValue)

                # draw arc
                startAngle = 216*16
                spanAngle  = -252*16*normValue

                if self.fCustomPaintMode == self.CUSTOM_PAINT_MODE_CARLA_WET:
                    painter.setBrush(colorBlue)
                    painter.setPen(QPen(colorBlue, 0))
                    painter.drawEllipse(QRectF(ballPoint.x(), ballPoint.y(), 2.2, 2.2))

                    gradient = QConicalGradient(15.5, 15.5, -45)
                    gradient.setColorAt(0.0,   colorBlue)
                    gradient.setColorAt(0.125, colorBlue)
                    gradient.setColorAt(0.625, colorGreen)
                    gradient.setColorAt(0.75,  colorGreen)
                    gradient.setColorAt(0.76,  colorGreen)
                    gradient.setColorAt(1.0,   colorGreen)
                    painter.setBrush(gradient)
                    painter.setPen(QPen(gradient, 3))

                else:
                    painter.setBrush(colorBlue)
                    painter.setPen(QPen(colorBlue, 0))
                    painter.drawEllipse(QRectF(ballPoint.x(), ballPoint.y(), 2.2, 2.2))

                    painter.setBrush(colorBlue)
                    painter.setPen(QPen(colorBlue, 3))

                painter.drawArc(4.0, 4.0, 26.0, 26.0, startAngle, spanAngle)

            # Custom knobs (L and R)
            elif self.fCustomPaintMode in (self.CUSTOM_PAINT_MODE_CARLA_L, self.CUSTOM_PAINT_MODE_CARLA_R):
                # knob color
                color = QColor(0xAD, 0xD5, 0x48).lighter(100 + self.fHoverStep*6)

                # draw small circle
                ballRect = QRectF(7.0, 8.0, 11.0, 12.0)
                ballPath = QPainterPath()
                ballPath.addEllipse(ballRect)
                #painter.drawRect(ballRect)
                tmpValue  = (0.375 + 0.75*normValue)
                ballValue = tmpValue - floor(tmpValue)
                ballPoint = ballPath.pointAtPercent(ballValue)

                painter.setBrush(color)
                painter.setPen(QPen(color, 0))
                painter.drawEllipse(QRectF(ballPoint.x(), ballPoint.y(), 2.0, 2.0))

                # draw arc
                if self.fCustomPaintMode == self.CUSTOM_PAINT_MODE_CARLA_L:
                    startAngle = 216*16
                    spanAngle  = -252.0*16*normValue
                else:
                    startAngle = 324.0*16
                    spanAngle  = 252.0*16*(1.0-normValue)

                painter.setPen(QPen(color, 2))
                painter.drawArc(3.5, 4.5, 22.0, 22.0, startAngle, spanAngle)

            # Custom knobs (Color)
            elif self.fCustomPaintMode == self.CUSTOM_PAINT_MODE_COLOR:
                # knob color
                color = self.fCustomPaintColor.lighter(100 + self.fHoverStep*6)

                # draw small circle
                ballRect = QRectF(8.0, 8.0, 15.0, 15.0)
                ballPath = QPainterPath()
                ballPath.addEllipse(ballRect)
                tmpValue  = (0.375 + 0.75*normValue)
                ballValue = tmpValue - floor(tmpValue)
                ballPoint = ballPath.pointAtPercent(ballValue)

                # draw arc
                startAngle = 216*16
                spanAngle  = -252*16*normValue

                painter.setBrush(color)
                painter.setPen(QPen(color, 0))
                painter.drawEllipse(QRectF(ballPoint.x(), ballPoint.y(), 2.2, 2.2))

                painter.setBrush(color)
                painter.setPen(QPen(color, 3))
                painter.drawArc(4.0, 4.0, 26.0, 26.0, startAngle, spanAngle)

            # Custom knobs (Zita)
            elif self.fCustomPaintMode == self.CUSTOM_PAINT_MODE_ZITA:
                a = normValue * pi * 1.5 - 2.35
                r = 10.0
                x = 10.5
                y = 10.5
                x += r * sin(a)
                y -= r * cos(a)
                painter.setBrush(Qt.black)
                painter.setPen(QPen(Qt.black, 2))
                painter.drawLine(QPointF(11.0, 11.0), QPointF(x, y))

            # Custom knobs
            else:
                painter.restore()
                return

            if self.HOVER_MIN < self.fHoverStep < self.HOVER_MAX:
                self.fHoverStep += 1 if self.fIsHovered else -1
                QTimer.singleShot(20, self.update)

        else: # isEnabled()
            target = QRectF(0.0, 0.0, self.fPixmapBaseSize, self.fPixmapBaseSize)
            painter.drawPixmap(target, self.fPixmap, target)

# ---------------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Carla Backend code (Qt stuff)
# Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ------------------------------------------------------------------------------------------------------------
# Imports (Global)

from PyQt5.QtCore import pyqtSignal, QObject

# ------------------------------------------------------------------------------------------------------------
# Imports (Custom)

from carla_backend import CarlaHostNull, CarlaHostDLL, CarlaHostPlugin

# ------------------------------------------------------------------------------------------------------------
# Carla Host PyQt signals

class CarlaHostSignals(QObject):
    # signals
    DebugCallback = pyqtSignal(int, int, int, int, float, str)
    PluginAddedCallback = pyqtSignal(int, int, str)
    PluginRemovedCallback = pyqtSignal(int)
    PluginRenamedCallback = pyqtSignal(int, str)
    PluginUnavailableCallback = pyqtSignal(int, str)
    ParameterValueChangedCallback = pyqtSignal(int, int, float)
    ParameterDefaultChangedCallback = pyqtSignal(int, int, float)
    ParameterMappedControlIndexChangedCallback = pyqtSignal(int, int, int)
    ParameterMappedRangeChangedCallback = pyqtSignal(int, int, float, float)
    ParameterMidiChannelChangedCallback = pyqtSignal(int, int, int)
    ProgramChangedCallback = pyqtSignal(int, int)
    MidiProgramChangedCallback = pyqtSignal(int, int)
    OptionChangedCallback = pyqtSignal(int, int, bool)
    UiStateChangedCallback = pyqtSignal(int, int)
    NoteOnCallback = pyqtSignal(int, int, int, int)
    NoteOffCallback = pyqtSignal(int, int, int)
    UpdateCallback = pyqtSignal(int)
    ReloadInfoCallback = pyqtSignal(int)
    ReloadParametersCallback = pyqtSignal(int)
    ReloadProgramsCallback = pyqtSignal(int)
    ReloadAllCallback = pyqtSignal(int)
    PatchbayClientAddedCallback = pyqtSignal(int, int, int, str)
    PatchbayClientRemovedCallback = pyqtSignal(int)
    PatchbayClientRenamedCallback = pyqtSignal(int, str)
    PatchbayClientDataChangedCallback = pyqtSignal(int, int, int)
    PatchbayClientPositionChangedCallback = pyqtSignal(int, int, int, int, int)
    PatchbayPortAddedCallback = pyqtSignal(int, int, int, int, str)
    PatchbayPortRemovedCallback = pyqtSignal(int, int)
    PatchbayPortChangedCallback = pyqtSignal(int, int, int, int, str)
    PatchbayPortGroupAddedCallback = pyqtSignal(int, int, int, str)
    PatchbayPortGroupRemovedCallback = pyqtSignal(int, int)
    PatchbayPortGroupChangedCallback = pyqtSignal(int, int, int, str)
    PatchbayConnectionAddedCallback = pyqtSignal(int, int, int, int, int)
    PatchbayConnectionRemovedCallback = pyqtSignal(int, int, int)
    EngineStartedCallback = pyqtSignal(int, int, int, int, float, str)
    EngineStoppedCallback = pyqtSignal()
    ProcessModeChangedCallback = pyqtSignal(int)
    TransportModeChangedCallback = pyqtSignal(int, str)
    BufferSizeChangedCallback = pyqtSignal(int)
    SampleRateChangedCallback = pyqtSignal(float)
    CancelableActionCallback = pyqtSignal(int, bool, str)
    ProjectLoadFinishedCallback = pyqtSignal()
    NSMCallback = pyqtSignal(int, int, str)
    InfoCallback = pyqtSignal(str)
    ErrorCallback = pyqtSignal(str)
    QuitCallback = pyqtSignal()
    InlineDisplayRedrawCallback = pyqtSignal(int)

# ------------------------------------------------------------------------------------------------------------
# Carla Host object (dummy/null, does nothing)

class CarlaHostQtNull(CarlaHostNull, CarlaHostSignals):
    def __init__(self):
        CarlaHostSignals.__init__(self)
        CarlaHostNull.__init__(self)

# ------------------------------------------------------------------------------------------------------------
# Carla Host object using a DLL

class CarlaHostQtDLL(CarlaHostDLL, CarlaHostSignals):
    def __init__(self, libName, loadGlobal):
        CarlaHostSignals.__init__(self)
        CarlaHostDLL.__init__(self, libName, loadGlobal)

# ------------------------------------------------------------------------------------------------------------
# Carla Host object for plugins (using pipes)

# pylint: disable=abstract-method
class CarlaHostQtPlugin(CarlaHostPlugin, CarlaHostSignals):
    def __init__(self):
        CarlaHostSignals.__init__(self)
        CarlaHostPlugin.__init__(self)

# pylint: enable=abstract-method
# ------------------------------------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_database.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginDatabaseW(object):
    def setupUi(self, PluginDatabaseW):
        PluginDatabaseW.setObjectName("PluginDatabaseW")
        PluginDatabaseW.resize(1100, 683)
        self.gridLayout_3 = QtWidgets.QGridLayout(PluginDatabaseW)
        self.gridLayout_3.setObjectName("gridLayout_3")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 2, 0, 1, 1)
        self.tab_reqs = QtWidgets.QTabWidget(PluginDatabaseW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tab_reqs.sizePolicy().hasHeightForWidth())
        self.tab_reqs.setSizePolicy(sizePolicy)
        self.tab_reqs.setProperty("tabBarAutoHide", True)
        self.tab_reqs.setObjectName("tab_reqs")
        self.tw_reqs = QtWidgets.QWidget()
        self.tw_reqs.setObjectName("tw_reqs")
        self.gridLayout = QtWidgets.QGridLayout(self.tw_reqs)
        self.gridLayout.setObjectName("gridLayout")
        self.l_reqs = QtWidgets.QLabel(self.tw_reqs)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.l_reqs.setFont(font)
        self.l_reqs.setObjectName("l_reqs")
        self.gridLayout.addWidget(self.l_reqs, 0, 1, 1, 1)
        self.ch_gui = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_gui.setObjectName("ch_gui")
        self.gridLayout.addWidget(self.ch_gui, 5, 1, 1, 1)
        self.ch_cv = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_cv.setObjectName("ch_cv")
        self.gridLayout.addWidget(self.ch_cv, 4, 1, 1, 1)
        self.ch_rtsafe = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_rtsafe.setObjectName("ch_rtsafe")
        self.gridLayout.addWidget(self.ch_rtsafe, 2, 1, 1, 1)
        self.ch_stereo = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_stereo.setObjectName("ch_stereo")
        self.gridLayout.addWidget(self.ch_stereo, 3, 1, 1, 1)
        self.ch_inline_display = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_inline_display.setObjectName("ch_inline_display")
        self.gridLayout.addWidget(self.ch_inline_display, 6, 1, 1, 1)
        self.ch_favorites = QtWidgets.QCheckBox(self.tw_reqs)
        self.ch_favorites.setObjectName("ch_favorites")
        self.gridLayout.addWidget(self.ch_favorites, 1, 1, 1, 1)
        self.tab_reqs.addTab(self.tw_reqs, "")
        self.gridLayout_3.addWidget(self.tab_reqs, 3, 0, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(PluginDatabaseW)
        self.label.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.b_add = QtWidgets.QPushButton(PluginDatabaseW)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/list-add.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_add.setIcon(icon)
        self.b_add.setObjectName("b_add")
        self.horizontalLayout_2.addWidget(self.b_add)
        self.b_cancel = QtWidgets.QPushButton(PluginDatabaseW)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/dialog-cancel.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_cancel.setIcon(icon1)
        self.b_cancel.setObjectName("b_cancel")
        self.horizontalLayout_2.addWidget(self.b_cancel)
        self.gridLayout_3.addLayout(self.horizontalLayout_2, 4, 0, 1, 3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdit = QtWidgets.QLineEdit(PluginDatabaseW)
        self.lineEdit.setProperty("clearButtonEnabled", True)
        self.lineEdit.setObjectName("lineEdit")
        self.horizontalLayout.addWidget(self.lineEdit)
        self.b_refresh = QtWidgets.QPushButton(PluginDatabaseW)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/16x16/view-refresh.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_refresh.setIcon(icon2)
        self.b_refresh.setObjectName("b_refresh")
        self.horizontalLayout.addWidget(self.b_refresh)
        self.b_clear_filters = QtWidgets.QPushButton(PluginDatabaseW)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/16x16/edit-clear.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_clear_filters.setIcon(icon3)
        self.b_clear_filters.setObjectName("b_clear_filters")
        self.horizontalLayout.addWidget(self.b_clear_filters)
        self.gridLayout_3.addLayout(self.horizontalLayout, 0, 0, 1, 3)
        self.tab_info = QtWidgets.QTabWidget(PluginDatabaseW)
        self.tab_info.setProperty("tabBarAutoHide", True)
        self.tab_info.setObjectName("tab_info")
        self.tw_info = QtWidgets.QWidget()
        self.tw_info.setObjectName("tw_info")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tw_info)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem2, 19, 0, 1, 1)
        self.l_format = QtWidgets.QLabel(self.tw_info)
        self.l_format.setObjectName("l_format")
        self.gridLayout_2.addWidget(self.l_format, 1, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.tw_info)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 1, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.tw_info)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout_2.addWidget(self.label_4, 3, 0, 1, 1)
        self.l_arch = QtWidgets.QLabel(self.tw_info)
        self.l_arch.setObjectName("l_arch")
        self.gridLayout_2.addWidget(self.l_arch, 3, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.tw_info)
        self.label_6.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 2, 0, 1, 1)
        self.l_type = QtWidgets.QLabel(self.tw_info)
        self.l_type.setObjectName("l_type")
        self.gridLayout_2.addWidget(self.l_type, 2, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem3, 19, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.tw_info)
        self.label_12.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_12.setObjectName("label_12")
        self.gridLayout_2.addWidget(self.label_12, 10, 0, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.tw_info)
        self.label_8.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 6, 0, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.tw_info)
        self.label_11.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_11.setObjectName("label_11")
        self.gridLayout_2.addWidget(self.label_11, 9, 0, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.tw_info)
        self.label_13.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_13.setObjectName("label_13")
        self.gridLayout_2.addWidget(self.label_13, 11, 0, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.tw_info)
        self.label_14.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 12, 0, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.tw_info)
        self.label_15.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_15.setObjectName("label_15")
        self.gridLayout_2.addWidget(self.label_15, 13, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.tw_info)
        self.label_9.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 7, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.tw_info)
        self.label_10.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 8, 0, 1, 1)
        self.la_id = QtWidgets.QLabel(self.tw_info)
        self.la_id.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.la_id.setObjectName("la_id")
        self.gridLayout_2.addWidget(self.la_id, 4, 0, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.tw_info)
        self.label_18.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_18.setObjectName("label_18")
        self.gridLayout_2.addWidget(self.label_18, 16, 0, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.tw_info)
        self.label_17.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_17.setObjectName("label_17")
        self.gridLayout_2.addWidget(self.label_17, 15, 0, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.tw_info)
        self.label_19.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_19.setObjectName("label_19")
        self.gridLayout_2.addWidget(self.label_19, 18, 0, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.tw_info)
        self.label_20.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_20.setObjectName("label_20")
        self.gridLayout_2.addWidget(self.label_20, 17, 0, 1, 1)
        self.l_aouts = QtWidgets.QLabel(self.tw_info)
        self.l_aouts.setObjectName("l_aouts")
        self.gridLayout_2.addWidget(self.l_aouts, 7, 1, 1, 1)
        self.l_id = QtWidgets.QLabel(self.tw_info)
        self.l_id.setObjectName("l_id")
        self.gridLayout_2.addWidget(self.l_id, 4, 1, 1, 1)
        self.l_ains = QtWidgets.QLabel(self.tw_info)
        self.l_ains.setObjectName("l_ains")
        self.gridLayout_2.addWidget(self.l_ains, 6, 1, 1, 1)
        self.l_synth = QtWidgets.QLabel(self.tw_info)
        self.l_synth.setObjectName("l_synth")
        self.gridLayout_2.addWidget(self.l_synth, 18, 1, 1, 1)
        self.l_pins = QtWidgets.QLabel(self.tw_info)
        self.l_pins.setObjectName("l_pins")
        self.gridLayout_2.addWidget(self.l_pins, 12, 1, 1, 1)
        self.l_cvins = QtWidgets.QLabel(self.tw_info)
        self.l_cvins.setObjectName("l_cvins")
        self.gridLayout_2.addWidget(self.l_cvins, 8, 1, 1, 1)
        self.l_pouts = QtWidgets.QLabel(self.tw_info)
        self.l_pouts.setObjectName("l_pouts")
        self.gridLayout_2.addWidget(self.l_pouts, 13, 1, 1, 1)
        self.l_mouts = QtWidgets.QLabel(self.tw_info)
        self.l_mouts.setObjectName("l_mouts")
        self.gridLayout_2.addWidget(self.l_mouts, 11, 1, 1, 1)
        self.l_mins = QtWidgets.QLabel(self.tw_info)
        self.l_mins.setObjectName("l_mins")
        self.gridLayout_2.addWidget(self.l_mins, 10, 1, 1, 1)
        self.l_bridged = QtWidgets.QLabel(self.tw_info)
        self.l_bridged.setObjectName("l_bridged")
        self.gridLayout_2.addWidget(self.l_bridged, 17, 1, 1, 1)
        self.l_idisp = QtWidgets.QLabel(self.tw_info)
        self.l_idisp.setObjectName("l_idisp")
        self.gridLayout_2.addWidget(self.l_idisp, 16, 1, 1, 1)
        self.l_gui = QtWidgets.QLabel(self.tw_info)
        self.l_gui.setObjectName("l_gui")
        self.gridLayout_2.addWidget(self.l_gui, 15, 1, 1, 1)
        self.l_cvouts = QtWidgets.QLabel(self.tw_info)
        self.l_cvouts.setObjectName("l_cvouts")
        self.gridLayout_2.addWidget(self.l_cvouts, 9, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.tw_info)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 0, 0, 1, 2)
        self.line_2 = QtWidgets.QFrame(self.tw_info)
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.gridLayout_2.addWidget(self.line_2, 14, 0, 1, 2)
        self.line = QtWidgets.QFrame(self.tw_info)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_2.addWidget(self.line, 5, 0, 1, 2)
        self.tab_info.addTab(self.tw_info, "")
        self.gridLayout_3.addWidget(self.tab_info, 1, 2, 3, 1)
        self.tableWidget = QtWidgets.QTableWidget(PluginDatabaseW)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tableWidget.sizePolicy().hasHeightForWidth())
        self.tableWidget.setSizePolicy(sizePolicy)
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setProperty("showDropIndicator", False)
        self.tableWidget.setDragDropOverwriteMode(False)
        self.tableWidget.setAlternatingRowColors(True)
        self.tableWidget.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableWidget.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableWidget.setShowGrid(False)
        self.tableWidget.setGridStyle(QtCore.Qt.NoPen)
        self.tableWidget.setWordWrap(False)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(5)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/16x16/bookmarks.svgz"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item.setIcon(icon4)
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(4, item)
        self.tableWidget.horizontalHeader().setMinimumSectionSize(24)
        self.tableWidget.horizontalHeader().setStretchLastSection(True)
        self.tableWidget.verticalHeader().setVisible(False)
        self.tableWidget.verticalHeader().setDefaultSectionSize(22)
        self.tableWidget.verticalHeader().setMinimumSectionSize(12)
        self.gridLayout_3.addWidget(self.tableWidget, 1, 1, 3, 1)
        self.toolBox = QtWidgets.QToolBox(PluginDatabaseW)
        self.toolBox.setObjectName("toolBox")
        self.p_format = QtWidgets.QWidget()
        self.p_format.setGeometry(QtCore.QRect(0, 0, 158, 237))
        self.p_format.setObjectName("p_format")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.p_format)
        self.verticalLayout.setObjectName("verticalLayout")
        self.ch_internal = QtWidgets.QCheckBox(self.p_format)
        self.ch_internal.setObjectName("ch_internal")
        self.verticalLayout.addWidget(self.ch_internal)
        self.ch_ladspa = QtWidgets.QCheckBox(self.p_format)
        self.ch_ladspa.setObjectName("ch_ladspa")
        self.verticalLayout.addWidget(self.ch_ladspa)
        self.ch_dssi = QtWidgets.QCheckBox(self.p_format)
        self.ch_dssi.setObjectName("ch_dssi")
        self.verticalLayout.addWidget(self.ch_dssi)
        self.ch_lv2 = QtWidgets.QCheckBox(self.p_format)
        self.ch_lv2.setObjectName("ch_lv2")
        self.verticalLayout.addWidget(self.ch_lv2)
        self.ch_vst = QtWidgets.QCheckBox(self.p_format)
        self.ch_vst.setObjectName("ch_vst")
        self.verticalLayout.addWidget(self.ch_vst)
        self.ch_vst3 = QtWidgets.QCheckBox(self.p_format)
        self.ch_vst3.setObjectName("ch_vst3")
        self.verticalLayout.addWidget(self.ch_vst3)
        self.ch_au = QtWidgets.QCheckBox(self.p_format)
        self.ch_au.setObjectName("ch_au")
        self.verticalLayout.addWidget(self.ch_au)
        self.ch_jsfx = QtWidgets.QCheckBox(self.p_format)
        self.ch_jsfx.setObjectName("ch_jsfx")
        self.verticalLayout.addWidget(self.ch_jsfx)
        self.ch_kits = QtWidgets.QCheckBox(self.p_format)
        self.ch_kits.setObjectName("ch_kits")
        self.verticalLayout.addWidget(self.ch_kits)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem4)
        self.toolBox.addItem(self.p_format, "")
        self.p_type = QtWidgets.QWidget()
        self.p_type.setGeometry(QtCore.QRect(0, 0, 158, 156))
        self.p_type.setObjectName("p_type")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.p_type)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.ch_effects = QtWidgets.QCheckBox(self.p_type)
        self.ch_effects.setObjectName("ch_effects")
        self.verticalLayout_2.addWidget(self.ch_effects)
        self.ch_instruments = QtWidgets.QCheckBox(self.p_type)
        self.ch_instruments.setObjectName("ch_instruments")
        self.verticalLayout_2.addWidget(self.ch_instruments)
        self.ch_midi = QtWidgets.QCheckBox(self.p_type)
        self.ch_midi.setObjectName("ch_midi")
        self.verticalLayout_2.addWidget(self.ch_midi)
        self.ch_other = QtWidgets.QCheckBox(self.p_type)
        self.ch_other.setObjectName("ch_other")
        self.verticalLayout_2.addWidget(self.ch_other)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem5)
        self.toolBox.addItem(self.p_type, "")
        self.p_category = QtWidgets.QWidget()
        self.p_category.setObjectName("p_category")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.p_category)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ch_cat_all = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_all.setObjectName("ch_cat_all")
        self.verticalLayout_4.addWidget(self.ch_cat_all)
        self.ch_cat_delay = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_delay.setObjectName("ch_cat_delay")
        self.verticalLayout_4.addWidget(self.ch_cat_delay)
        self.ch_cat_distortion = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_distortion.setObjectName("ch_cat_distortion")
        self.verticalLayout_4.addWidget(self.ch_cat_distortion)
        self.ch_cat_dynamics = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_dynamics.setObjectName("ch_cat_dynamics")
        self.verticalLayout_4.addWidget(self.ch_cat_dynamics)
        self.ch_cat_eq = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_eq.setObjectName("ch_cat_eq")
        self.verticalLayout_4.addWidget(self.ch_cat_eq)
        self.ch_cat_filter = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_filter.setObjectName("ch_cat_filter")
        self.verticalLayout_4.addWidget(self.ch_cat_filter)
        self.ch_cat_modulator = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_modulator.setObjectName("ch_cat_modulator")
        self.verticalLayout_4.addWidget(self.ch_cat_modulator)
        self.ch_cat_synth = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_synth.setObjectName("ch_cat_synth")
        self.verticalLayout_4.addWidget(self.ch_cat_synth)
        self.ch_cat_utility = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_utility.setObjectName("ch_cat_utility")
        self.verticalLayout_4.addWidget(self.ch_cat_utility)
        self.ch_cat_other = QtWidgets.QCheckBox(self.p_category)
        self.ch_cat_other.setObjectName("ch_cat_other")
        self.verticalLayout_4.addWidget(self.ch_cat_other)
        spacerItem6 = QtWidgets.QSpacerItem(20, 23, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem6)
        self.toolBox.addItem(self.p_category, "")
        self.p_arch = QtWidgets.QWidget()
        self.p_arch.setGeometry(QtCore.QRect(0, 0, 158, 130))
        self.p_arch.setObjectName("p_arch")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.p_arch)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.ch_native = QtWidgets.QCheckBox(self.p_arch)
        self.ch_native.setObjectName("ch_native")
        self.verticalLayout_3.addWidget(self.ch_native)
        self.ch_bridged = QtWidgets.QCheckBox(self.p_arch)
        self.ch_bridged.setObjectName("ch_bridged")
        self.verticalLayout_3.addWidget(self.ch_bridged)
        self.ch_bridged_wine = QtWidgets.QCheckBox(self.p_arch)
        self.ch_bridged_wine.setObjectName("ch_bridged_wine")
        self.verticalLayout_3.addWidget(self.ch_bridged_wine)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem7)
        self.toolBox.addItem(self.p_arch, "")
        self.gridLayout_3.addWidget(self.toolBox, 1, 0, 1, 1)
        self.act_focus_search = QtWidgets.QAction(PluginDatabaseW)
        self.act_focus_search.setObjectName("act_focus_search")

        self.retranslateUi(PluginDatabaseW)
        self.toolBox.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(PluginDatabaseW)
        PluginDatabaseW.setTabOrder(self.lineEdit, self.tableWidget)
        PluginDatabaseW.setTabOrder(self.tableWidget, self.b_add)
        PluginDatabaseW.setTabOrder(self.b_add, self.b_cancel)
        PluginDatabaseW.setTabOrder(self.b_cancel, self.b_refresh)
        PluginDatabaseW.setTabOrder(self.b_refresh, self.b_clear_filters)
        PluginDatabaseW.setTabOrder(self.b_clear_filters, self.ch_internal)
        PluginDatabaseW.setTabOrder(self.ch_internal, self.ch_ladspa)
        PluginDatabaseW.setTabOrder(self.ch_ladspa, self.ch_dssi)
        PluginDatabaseW.setTabOrder(self.ch_dssi, self.ch_lv2)
        PluginDatabaseW.setTabOrder(self.ch_lv2, self.ch_vst)
        PluginDatabaseW.setTabOrder(self.ch_vst, self.ch_vst3)
        PluginDatabaseW.setTabOrder(self.ch_vst3, self.ch_au)
        PluginDatabaseW.setTabOrder(self.ch_au, self.ch_kits)
        PluginDatabaseW.setTabOrder(self.ch_kits, self.ch_effects)
        PluginDatabaseW.setTabOrder(self.ch_effects, self.ch_instruments)
        PluginDatabaseW.setTabOrder(self.ch_instruments, self.ch_midi)
        PluginDatabaseW.setTabOrder(self.ch_midi, self.ch_other)
        PluginDatabaseW.setTabOrder(self.ch_other, self.ch_native)
        PluginDatabaseW.setTabOrder(self.ch_native, self.ch_bridged)
        PluginDatabaseW.setTabOrder(self.ch_bridged, self.ch_bridged_wine)
        PluginDatabaseW.setTabOrder(self.ch_bridged_wine, self.ch_inline_display)
        PluginDatabaseW.setTabOrder(self.ch_inline_display, self.ch_stereo)
        PluginDatabaseW.setTabOrder(self.ch_stereo, self.ch_rtsafe)
        PluginDatabaseW.setTabOrder(self.ch_rtsafe, self.ch_gui)
        PluginDatabaseW.setTabOrder(self.ch_gui, self.ch_cv)
        PluginDatabaseW.setTabOrder(self.ch_cv, self.tab_reqs)
        PluginDatabaseW.setTabOrder(self.tab_reqs, self.tab_info)

    def retranslateUi(self, PluginDatabaseW):
        _translate = QtCore.QCoreApplication.translate
        PluginDatabaseW.setWindowTitle(_translate("PluginDatabaseW", "Carla - Add New"))
        self.l_reqs.setText(_translate("PluginDatabaseW", "Requirements"))
        self.ch_gui.setText(_translate("PluginDatabaseW", "With Custom GUI "))
        self.ch_cv.setText(_translate("PluginDatabaseW", "With CV Ports"))
        self.ch_rtsafe.setText(_translate("PluginDatabaseW", "Real-time safe only"))
        self.ch_stereo.setText(_translate("PluginDatabaseW", "Stereo only"))
        self.ch_inline_display.setText(_translate("PluginDatabaseW", "With Inline Display"))
        self.ch_favorites.setText(_translate("PluginDatabaseW", "Favorites only"))
        self.label.setText(_translate("PluginDatabaseW", "(Number of Plugins go here)"))
        self.b_add.setText(_translate("PluginDatabaseW", "&Add Plugin"))
        self.b_cancel.setText(_translate("PluginDatabaseW", "Cancel"))
        self.b_refresh.setText(_translate("PluginDatabaseW", "Refresh"))
        self.b_clear_filters.setText(_translate("PluginDatabaseW", "Reset filters"))
        self.l_format.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_2.setText(_translate("PluginDatabaseW", "Format:"))
        self.label_4.setText(_translate("PluginDatabaseW", "Architecture:"))
        self.l_arch.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_6.setText(_translate("PluginDatabaseW", "Type:"))
        self.l_type.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_12.setText(_translate("PluginDatabaseW", "MIDI Ins:"))
        self.label_8.setText(_translate("PluginDatabaseW", "Audio Ins:"))
        self.label_11.setText(_translate("PluginDatabaseW", "CV Outs:"))
        self.label_13.setText(_translate("PluginDatabaseW", "MIDI Outs:"))
        self.label_14.setText(_translate("PluginDatabaseW", "Parameter Ins:"))
        self.label_15.setText(_translate("PluginDatabaseW", "Parameter Outs:"))
        self.label_9.setText(_translate("PluginDatabaseW", "Audio Outs:"))
        self.label_10.setText(_translate("PluginDatabaseW", "CV Ins:"))
        self.la_id.setText(_translate("PluginDatabaseW", "UniqueID:"))
        self.label_18.setText(_translate("PluginDatabaseW", "Has Inline Display:"))
        self.label_17.setText(_translate("PluginDatabaseW", "Has Custom GUI:"))
        self.label_19.setText(_translate("PluginDatabaseW", "Is Synth:"))
        self.label_20.setText(_translate("PluginDatabaseW", "Is Bridged:"))
        self.l_aouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_id.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_ains.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_synth.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_pins.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_cvins.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_pouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_mouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_mins.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_bridged.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_idisp.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_gui.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.l_cvouts.setText(_translate("PluginDatabaseW", "TextLabel"))
        self.label_3.setText(_translate("PluginDatabaseW", "Information"))
        self.tableWidget.setSortingEnabled(True)
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("PluginDatabaseW", "Name"))
        item = self.tableWidget.horizontalHeaderItem(2)
        item.setText(_translate("PluginDatabaseW", "Label/URI"))
        item = self.tableWidget.horizontalHeaderItem(3)
        item.setText(_translate("PluginDatabaseW", "Maker"))
        item = self.tableWidget.horizontalHeaderItem(4)
        item.setText(_translate("PluginDatabaseW", "Binary/Filename"))
        self.ch_internal.setText(_translate("PluginDatabaseW", "Internal"))
        self.ch_ladspa.setText(_translate("PluginDatabaseW", "LADSPA"))
        self.ch_dssi.setText(_translate("PluginDatabaseW", "DSSI"))
        self.ch_lv2.setText(_translate("PluginDatabaseW", "LV2"))
        self.ch_vst.setText(_translate("PluginDatabaseW", "VST2"))
        self.ch_vst3.setText(_translate("PluginDatabaseW", "VST3"))
        self.ch_au.setText(_translate("PluginDatabaseW", "AU"))
        self.ch_jsfx.setText(_translate("PluginDatabaseW", "JSFX"))
        self.ch_kits.setText(_translate("PluginDatabaseW", "Sound Kits"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_format), _translate("PluginDatabaseW", "Format"))
        self.ch_effects.setText(_translate("PluginDatabaseW", "Effects"))
        self.ch_instruments.setText(_translate("PluginDatabaseW", "Instruments"))
        self.ch_midi.setText(_translate("PluginDatabaseW", "MIDI Plugins"))
        self.ch_other.setText(_translate("PluginDatabaseW", "Other/Misc"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_type), _translate("PluginDatabaseW", "Type"))
        self.ch_cat_all.setText(_translate("PluginDatabaseW", "All"))
        self.ch_cat_delay.setText(_translate("PluginDatabaseW", "Delay"))
        self.ch_cat_distortion.setText(_translate("PluginDatabaseW", "Distortion"))
        self.ch_cat_dynamics.setText(_translate("PluginDatabaseW", "Dynamics"))
        self.ch_cat_eq.setText(_translate("PluginDatabaseW", "EQ"))
        self.ch_cat_filter.setText(_translate("PluginDatabaseW", "Filter"))
        self.ch_cat_modulator.setText(_translate("PluginDatabaseW", "Modulator"))
        self.ch_cat_synth.setText(_translate("PluginDatabaseW", "Synth"))
        self.ch_cat_utility.setText(_translate("PluginDatabaseW", "Utility"))
        self.ch_cat_other.setText(_translate("PluginDatabaseW", "Other"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_category), _translate("PluginDatabaseW", "Category"))
        self.ch_native.setText(_translate("PluginDatabaseW", "Native"))
        self.ch_bridged.setText(_translate("PluginDatabaseW", "Bridged"))
        self.ch_bridged_wine.setText(_translate("PluginDatabaseW", "Bridged (Wine)"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.p_arch), _translate("PluginDatabaseW", "Architecture"))
        self.act_focus_search.setText(_translate("PluginDatabaseW", "Focus Text Search"))
        self.act_focus_search.setShortcut(_translate("PluginDatabaseW", "Ctrl+F"))
import resources_rc



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# PatchBay Canvas engine using QGraphicsView/Scene
# Copyright (C) 2010-2021 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ------------------------------------------------------------------------------------------------------------
# Imports (Global)

from PyQt5.QtCore import pyqtSignal, pyqtSlot, qCritical, QT_VERSION, Qt, QPointF, QRectF, QTimer
from PyQt5.QtGui import QCursor, QFont, QFontMetrics, QImage, QLinearGradient, QPainter, QPen
from PyQt5.QtWidgets import QGraphicsItem, QGraphicsObject, QMenu

# ------------------------------------------------------------------------------------------------------------
# Backwards-compatible horizontalAdvance/width call, depending on Qt version

def fontHorizontalAdvance(font, string):
    if QT_VERSION >= 0x50b00:
        return QFontMetrics(font).horizontalAdvance(string)
    return QFontMetrics(font).width(string)

# ------------------------------------------------------------------------------------------------------------
# Imports (Custom)

from . import (
    canvas,
    features,
    options,
    port_dict_t,
    CanvasBoxType,
    ANTIALIASING_FULL,
    ACTION_PLUGIN_EDIT,
    ACTION_PLUGIN_SHOW_UI,
    ACTION_PLUGIN_CLONE,
    ACTION_PLUGIN_REMOVE,
    ACTION_PLUGIN_RENAME,
    ACTION_PLUGIN_REPLACE,
    ACTION_GROUP_INFO,
    ACTION_GROUP_JOIN,
    ACTION_GROUP_SPLIT,
    ACTION_GROUP_RENAME,
    ACTION_PORTS_DISCONNECT,
    ACTION_INLINE_DISPLAY,
    EYECANDY_FULL,
    PORT_MODE_NULL,
    PORT_MODE_INPUT,
    PORT_MODE_OUTPUT,
    PORT_TYPE_NULL,
    PORT_TYPE_AUDIO_JACK,
    PORT_TYPE_MIDI_ALSA,
    PORT_TYPE_MIDI_JACK,
    PORT_TYPE_PARAMETER,
    MAX_PLUGIN_ID_ALLOWED,
)

from .canvasboxshadow import CanvasBoxShadow
from .canvasicon import CanvasIcon
from .canvasport import CanvasPort
from .theme import Theme
from .utils import CanvasItemFX, CanvasGetFullPortName, CanvasGetPortConnectionList

# ------------------------------------------------------------------------------------------------------------

class cb_line_t(object):
    def __init__(self, line, connection_id):
        self.line = line
        self.connection_id = connection_id

# ------------------------------------------------------------------------------------------------------------

class CanvasBox(QGraphicsObject):
    # signals
    positionChanged = pyqtSignal(int, bool, int, int)

    # enums
    INLINE_DISPLAY_DISABLED = 0
    INLINE_DISPLAY_ENABLED  = 1
    INLINE_DISPLAY_CACHED   = 2

    def __init__(self, group_id, group_name, icon, parent=None):
        QGraphicsObject.__init__(self)
        self.setParentItem(parent)

        # Save Variables, useful for later
        self.m_group_id = group_id
        self.m_group_name = group_name

        # plugin Id, < 0 if invalid
        self.m_plugin_id = -1
        self.m_plugin_ui = False
        self.m_plugin_inline = self.INLINE_DISPLAY_DISABLED

        # Base Variables
        self.p_width = 50
        self.p_width_in = 0
        self.p_width_out = 0
        self.p_height = canvas.theme.box_header_height + canvas.theme.box_header_spacing + 1

        self.m_last_pos = QPointF()
        self.m_split = False
        self.m_split_mode = PORT_MODE_NULL

        self.m_cursor_moving = False
        self.m_forced_split = False
        self.m_mouse_down = False
        self.m_inline_image = None
        self.m_inline_scaling = 1.0
        self.m_inline_first = True
        self.m_will_signal_pos_change = False

        self.m_port_list_ids = []
        self.m_connection_lines = []

        # Set Font
        self.m_font_name = QFont()
        self.m_font_name.setFamily(canvas.theme.box_font_name)
        self.m_font_name.setPixelSize(canvas.theme.box_font_size)
        self.m_font_name.setWeight(canvas.theme.box_font_state)

        self.m_font_port = QFont()
        self.m_font_port.setFamily(canvas.theme.port_font_name)
        self.m_font_port.setPixelSize(canvas.theme.port_font_size)
        self.m_font_port.setWeight(canvas.theme.port_font_state)

        # Icon
        if canvas.theme.box_use_icon:
            self.icon_svg = CanvasIcon(icon, self.m_group_name, self)
        else:
            self.icon_svg = None

        # Shadow
        if options.eyecandy and QT_VERSION >= 0x50c00:
            self.shadow = CanvasBoxShadow(self.toGraphicsObject())
            self.shadow.setFakeParent(self)
            self.setGraphicsEffect(self.shadow)
        else:
            self.shadow = None

        # Final touches
        self.setFlags(QGraphicsItem.ItemIsFocusable | QGraphicsItem.ItemIsMovable | QGraphicsItem.ItemIsSelectable)

        # Wait for at least 1 port
        if options.auto_hide_groups:
            self.setVisible(False)

        if options.auto_select_items:
            self.setAcceptHoverEvents(True)

        self.updatePositions()

        self.visibleChanged.connect(self.slot_signalPositionChangedLater)
        self.xChanged.connect(self.slot_signalPositionChangedLater)
        self.yChanged.connect(self.slot_signalPositionChangedLater)

        canvas.scene.addItem(self)
        QTimer.singleShot(0, self.fixPos)

    def getGroupId(self):
        return self.m_group_id

    def getGroupName(self):
        return self.m_group_name

    def isSplit(self):
        return self.m_split

    def getSplitMode(self):
        return self.m_split_mode

    def getPortCount(self):
        return len(self.m_port_list_ids)

    def getPortList(self):
        return self.m_port_list_ids

    def redrawInlineDisplay(self):
        if self.m_plugin_inline == self.INLINE_DISPLAY_CACHED:
            self.m_plugin_inline = self.INLINE_DISPLAY_ENABLED
            self.update()

    def removeAsPlugin(self):
        #del self.m_inline_image
        #self.m_inline_image = None
        #self.m_inline_scaling = 1.0

        self.m_plugin_id = -1
        self.m_plugin_ui = False
        self.m_plugin_inline = self.INLINE_DISPLAY_DISABLED

    def setAsPlugin(self, plugin_id, hasUI, hasInlineDisplay):
        if hasInlineDisplay and not options.inline_displays:
            hasInlineDisplay = False

        if not hasInlineDisplay:
            self.m_inline_image = None
            self.m_inline_scaling = 1.0

        self.m_plugin_id = plugin_id
        self.m_plugin_ui = hasUI
        self.m_plugin_inline = self.INLINE_DISPLAY_ENABLED if hasInlineDisplay else self.INLINE_DISPLAY_DISABLED
        self.update()

    def setIcon(self, icon):
        if self.icon_svg is not None:
            self.icon_svg.setIcon(icon, self.m_group_name)

    def setSplit(self, split, mode=PORT_MODE_NULL):
        self.m_split = split
        self.m_split_mode = mode

    def setGroupName(self, group_name):
        self.m_group_name = group_name
        self.updatePositions()

    def setShadowOpacity(self, opacity):
        if self.shadow is not None:
            self.shadow.setOpacity(opacity)

    def addPortFromGroup(self, port_id, port_mode, port_type, port_name, is_alternate):
        if len(self.m_port_list_ids) == 0:
            if options.auto_hide_groups:
                if options.eyecandy == EYECANDY_FULL:
                    CanvasItemFX(self, True, False)
                self.blockSignals(True)
                self.setVisible(True)
                self.blockSignals(False)

        new_widget = CanvasPort(self.m_group_id, port_id, port_name, port_mode, port_type, is_alternate, self)

        port_dict = port_dict_t()
        port_dict.group_id = self.m_group_id
        port_dict.port_id = port_id
        port_dict.port_name = port_name
        port_dict.port_mode = port_mode
        port_dict.port_type = port_type
        port_dict.is_alternate = is_alternate
        port_dict.widget = new_widget

        self.m_port_list_ids.append(port_id)

        return new_widget

    def removePortFromGroup(self, port_id):
        if port_id in self.m_port_list_ids:
            self.m_port_list_ids.remove(port_id)
        else:
            qCritical("PatchCanvas::CanvasBox.removePort(%i) - unable to find port to remove" % port_id)
            return

        if len(self.m_port_list_ids) > 0:
            self.updatePositions()

        elif self.isVisible():
            if options.auto_hide_groups:
                if options.eyecandy == EYECANDY_FULL:
                    CanvasItemFX(self, False, False)
                else:
                    self.blockSignals(True)
                    self.setVisible(False)
                    self.blockSignals(False)

    def addLineFromGroup(self, line, connection_id):
        new_cbline = cb_line_t(line, connection_id)
        self.m_connection_lines.append(new_cbline)

    def removeLineFromGroup(self, connection_id):
        for connection in self.m_connection_lines:
            if connection.connection_id == connection_id:
                self.m_connection_lines.remove(connection)
                return
        qCritical("PatchCanvas::CanvasBox.removeLineFromGroup(%i) - unable to find line to remove" % connection_id)

    def checkItemPos(self):
        if canvas.size_rect.isNull():
            return

        pos = self.scenePos()
        if (canvas.size_rect.contains(pos) and
            canvas.size_rect.contains(pos + QPointF(self.p_width, self.p_height))):
            return

        if pos.x() < canvas.size_rect.x():
            self.setPos(canvas.size_rect.x(), pos.y())
        elif pos.x() + self.p_width > canvas.size_rect.width():
            self.setPos(canvas.size_rect.width() - self.p_width, pos.y())

        pos = self.scenePos()
        if pos.y() < canvas.size_rect.y():
            self.setPos(pos.x(), canvas.size_rect.y())
        elif pos.y() + self.p_height > canvas.size_rect.height():
            self.setPos(pos.x(), canvas.size_rect.height() - self.p_height)

    def removeIconFromScene(self):
        if self.icon_svg is None:
            return

        item = self.icon_svg
        self.icon_svg = None
        canvas.scene.removeItem(item)
        del item

    def updatePositions(self):
        self.prepareGeometryChange()

        # Check Text Name size
        app_name_size = fontHorizontalAdvance(self.m_font_name, self.m_group_name) + 30
        self.p_width = max(50, app_name_size)

        # Get Port List
        port_list = []
        for port in canvas.port_list:
            if port.group_id == self.m_group_id and port.port_id in self.m_port_list_ids:
                port_list.append(port)

        if len(port_list) == 0:
            self.p_height = canvas.theme.box_header_height
            self.p_width_in = 0
            self.p_width_out = 0
        else:
            max_in_width = max_out_width = 0
            port_spacing = canvas.theme.port_height + canvas.theme.port_spacing

            # Get Max Box Width, vertical ports re-positioning
            port_types = (PORT_TYPE_AUDIO_JACK, PORT_TYPE_MIDI_JACK, PORT_TYPE_MIDI_ALSA, PORT_TYPE_PARAMETER)
            last_in_type = last_out_type = PORT_TYPE_NULL
            last_in_pos = last_out_pos = canvas.theme.box_header_height + canvas.theme.box_header_spacing

            for port_type in port_types:
                for port in port_list:
                    if port.port_type != port_type:
                        continue

                    size = fontHorizontalAdvance(self.m_font_port, port.port_name)

                    if port.port_mode == PORT_MODE_INPUT:
                        max_in_width = max(max_in_width, size)
                        if port.port_type != last_in_type:
                            if last_in_type != PORT_TYPE_NULL:
                                last_in_pos += canvas.theme.port_spacingT
                            last_in_type = port.port_type
                        port.widget.setY(last_in_pos)
                        last_in_pos += port_spacing

                    elif port.port_mode == PORT_MODE_OUTPUT:
                        max_out_width = max(max_out_width, size)
                        if port.port_type != last_out_type:
                            if last_out_type != PORT_TYPE_NULL:
                                last_out_pos += canvas.theme.port_spacingT
                            last_out_type = port.port_type
                        port.widget.setY(last_out_pos)
                        last_out_pos += port_spacing

            self.p_width     = max(self.p_width, 30 + max_in_width + max_out_width)
            self.p_width_in  = max_in_width
            self.p_width_out = max_out_width

            self.p_height  = max(last_in_pos, last_out_pos)
            self.p_height += max(canvas.theme.port_spacing, canvas.theme.port_spacingT) - canvas.theme.port_spacing
            self.p_height += canvas.theme.box_pen.width()

            self.repositionPorts(port_list)

        self.repaintLines(True)
        self.update()

    def repositionPorts(self, port_list = None):
        if port_list is None:
            port_list = []
            for port in canvas.port_list:
                if port.group_id == self.m_group_id and port.port_id in self.m_port_list_ids:
                    port_list.append(port)

        # Horizontal ports re-positioning
        inX = canvas.theme.port_offset
        outX = self.p_width - self.p_width_out - canvas.theme.port_offset - 12
        for port in port_list:
            if port.port_mode == PORT_MODE_INPUT:
                port.widget.setX(inX)
                port.widget.setPortWidth(self.p_width_in)

            elif port.port_mode == PORT_MODE_OUTPUT:
                port.widget.setX(outX)
                port.widget.setPortWidth(self.p_width_out)

    def repaintLines(self, forced=False):
        if self.pos() != self.m_last_pos or forced:
            for connection in self.m_connection_lines:
                connection.line.updateLinePos()

        self.m_last_pos = self.pos()

    def resetLinesZValue(self):
        for connection in canvas.connection_list:
            if connection.port_out_id in self.m_port_list_ids and connection.port_in_id in self.m_port_list_ids:
                z_value = canvas.last_z_value
            else:
                z_value = canvas.last_z_value - 1

            connection.widget.setZValue(z_value)

    def triggerSignalPositionChanged(self):
        self.positionChanged.emit(self.m_group_id, self.m_split, int(self.x()), int(self.y()))
        self.m_will_signal_pos_change = False

    @pyqtSlot()
    def slot_signalPositionChangedLater(self):
        if self.m_will_signal_pos_change:
            return
        self.m_will_signal_pos_change = True
        QTimer.singleShot(0, self.triggerSignalPositionChanged)

    def type(self):
        return CanvasBoxType

    def contextMenuEvent(self, event):
        event.accept()
        menu = QMenu()

        # Conenct menu stuff
        connMenu = QMenu("Connect", menu)

        our_port_types = []
        our_port_outs = {
            PORT_TYPE_AUDIO_JACK: [],
            PORT_TYPE_MIDI_JACK: [],
            PORT_TYPE_MIDI_ALSA: [],
            PORT_TYPE_PARAMETER: [],
        }
        for port in canvas.port_list:
            if port.group_id != self.m_group_id:
                continue
            if port.port_mode != PORT_MODE_OUTPUT:
                continue
            if port.port_id not in self.m_port_list_ids:
                continue
            if port.port_type not in our_port_types:
                our_port_types.append(port.port_type)
            our_port_outs[port.port_type].append((port.group_id, port.port_id))

        if len(our_port_types) != 0:
            act_x_conn = None
            for group in canvas.group_list:
                if self.m_group_id == group.group_id:
                    continue

                has_ports = False
                target_ports = {
                    PORT_TYPE_AUDIO_JACK: [],
                    PORT_TYPE_MIDI_JACK: [],
                    PORT_TYPE_MIDI_ALSA: [],
                    PORT_TYPE_PARAMETER: [],
                }

                for port in canvas.port_list:
                    if port.group_id != group.group_id:
                        continue
                    if port.port_mode != PORT_MODE_INPUT:
                        continue
                    if port.port_type not in our_port_types:
                        continue
                    has_ports = True
                    target_ports[port.port_type].append((port.group_id, port.port_id))

                if not has_ports:
                    continue

                act_x_conn = connMenu.addAction(group.group_name)
                act_x_conn.setData((our_port_outs, target_ports))
                act_x_conn.triggered.connect(canvas.qobject.PortContextMenuConnect)

            if act_x_conn is None:
                act_x_disc = connMenu.addAction("Nothing to connect to")
                act_x_disc.setEnabled(False)

        else:
            act_x_disc = connMenu.addAction("No output ports")
            act_x_disc.setEnabled(False)

        # Disconnect menu stuff
        discMenu = QMenu("Disconnect", menu)

        conn_list = []
        conn_list_ids = []

        for port_id in self.m_port_list_ids:
            tmp_conn_list = CanvasGetPortConnectionList(self.m_group_id, port_id)
            for tmp_conn_id, tmp_group_id, tmp_port_id in tmp_conn_list:
                if tmp_conn_id not in conn_list_ids:
                    conn_list.append((tmp_conn_id, tmp_group_id, tmp_port_id))
                    conn_list_ids.append(tmp_conn_id)

        if len(conn_list) > 0:
            for conn_id, group_id, port_id in conn_list:
                act_x_disc = discMenu.addAction(CanvasGetFullPortName(group_id, port_id))
                act_x_disc.setData(conn_id)
                act_x_disc.triggered.connect(canvas.qobject.PortContextMenuDisconnect)
        else:
            act_x_disc = discMenu.addAction("No connections")
            act_x_disc.setEnabled(False)

        menu.addMenu(connMenu)
        menu.addMenu(discMenu)
        act_x_disc_all = menu.addAction("Disconnect &All")
        act_x_sep1 = menu.addSeparator()
        act_x_info = menu.addAction("Info")
        act_x_rename = menu.addAction("Rename")
        act_x_sep2 = menu.addSeparator()
        act_x_split_join = menu.addAction("Join" if self.m_split else "Split")

        if not features.group_info:
            act_x_info.setVisible(False)

        if not features.group_rename:
            act_x_rename.setVisible(False)

        if not (features.group_info and features.group_rename):
            act_x_sep1.setVisible(False)

        if self.m_plugin_id >= 0 and self.m_plugin_id <= MAX_PLUGIN_ID_ALLOWED:
            menu.addSeparator()
            act_p_edit = menu.addAction("Edit")
            act_p_ui = menu.addAction("Show Custom UI")
            menu.addSeparator()
            act_p_clone = menu.addAction("Clone")
            act_p_rename = menu.addAction("Rename...")
            act_p_replace = menu.addAction("Replace...")
            act_p_remove = menu.addAction("Remove")

            if not self.m_plugin_ui:
                act_p_ui.setVisible(False)

        else:
            act_p_edit = act_p_ui = None
            act_p_clone = act_p_rename = None
            act_p_replace = act_p_remove = None

        haveIns = haveOuts = False
        for port in canvas.port_list:
            if port.group_id == self.m_group_id and port.port_id in self.m_port_list_ids:
                if port.port_mode == PORT_MODE_INPUT:
                    haveIns = True
                elif port.port_mode == PORT_MODE_OUTPUT:
                    haveOuts = True

        if not (self.m_split or bool(haveIns and haveOuts)):
            act_x_sep2.setVisible(False)
            act_x_split_join.setVisible(False)

        act_selected = menu.exec_(event.screenPos())

        if act_selected is None:
            pass

        elif act_selected == act_x_disc_all:
            for conn_id in conn_list_ids:
                canvas.callback(ACTION_PORTS_DISCONNECT, conn_id, 0, "")

        elif act_selected == act_x_info:
            canvas.callback(ACTION_GROUP_INFO, self.m_group_id, 0, "")

        elif act_selected == act_x_rename:
            canvas.callback(ACTION_GROUP_RENAME, self.m_group_id, 0, "")

        elif act_selected == act_x_split_join:
            if self.m_split:
                canvas.callback(ACTION_GROUP_JOIN, self.m_group_id, 0, "")
            else:
                canvas.callback(ACTION_GROUP_SPLIT, self.m_group_id, 0, "")

        elif act_selected == act_p_edit:
            canvas.callback(ACTION_PLUGIN_EDIT, self.m_plugin_id, 0, "")

        elif act_selected == act_p_ui:
            canvas.callback(ACTION_PLUGIN_SHOW_UI, self.m_plugin_id, 0, "")

        elif act_selected == act_p_clone:
            canvas.callback(ACTION_PLUGIN_CLONE, self.m_plugin_id, 0, "")

        elif act_selected == act_p_rename:
            canvas.callback(ACTION_PLUGIN_RENAME, self.m_plugin_id, 0, "")

        elif act_selected == act_p_replace:
            canvas.callback(ACTION_PLUGIN_REPLACE, self.m_plugin_id, 0, "")

        elif act_selected == act_p_remove:
            canvas.callback(ACTION_PLUGIN_REMOVE, self.m_plugin_id, 0, "")

    def keyPressEvent(self, event):
        if self.m_plugin_id >= 0 and event.key() == Qt.Key_Delete:
            event.accept()
            canvas.callback(ACTION_PLUGIN_REMOVE, self.m_plugin_id, 0, "")
            return
        QGraphicsObject.keyPressEvent(self, event)

    def hoverEnterEvent(self, event):
        if options.auto_select_items:
            if len(canvas.scene.selectedItems()) > 0:
                canvas.scene.clearSelection()
            self.setSelected(True)
        QGraphicsObject.hoverEnterEvent(self, event)

    def mouseDoubleClickEvent(self, event):
        if self.m_plugin_id >= 0:
            event.accept()
            canvas.callback(ACTION_PLUGIN_SHOW_UI if self.m_plugin_ui else ACTION_PLUGIN_EDIT, self.m_plugin_id, 0, "")
            return

        QGraphicsObject.mouseDoubleClickEvent(self, event)

    def mousePressEvent(self, event):
        if event.button() == Qt.MiddleButton or event.source() == Qt.MouseEventSynthesizedByApplication:
            event.ignore()
            return

        canvas.last_z_value += 1
        self.setZValue(canvas.last_z_value)
        self.resetLinesZValue()
        self.m_cursor_moving = False

        if event.button() == Qt.RightButton:
            event.accept()
            canvas.scene.clearSelection()
            self.setSelected(True)
            self.m_mouse_down = False
            return

        elif event.button() == Qt.LeftButton:
            if self.sceneBoundingRect().contains(event.scenePos()):
                self.m_mouse_down = True
            else:
                # FIXME: Check if still valid: Fix a weird Qt behaviour with right-click mouseMove
                self.m_mouse_down = False
                event.ignore()
                return

        else:
            self.m_mouse_down = False

        QGraphicsObject.mousePressEvent(self, event)

    def mouseMoveEvent(self, event):
        if self.m_mouse_down:
            if not self.m_cursor_moving:
                self.setCursor(QCursor(Qt.SizeAllCursor))
                self.m_cursor_moving = True
            self.repaintLines()
        QGraphicsObject.mouseMoveEvent(self, event)

    def mouseReleaseEvent(self, event):
        if self.m_cursor_moving:
            self.unsetCursor()
            QTimer.singleShot(0, self.fixPos)
        self.m_mouse_down = False
        self.m_cursor_moving = False
        QGraphicsObject.mouseReleaseEvent(self, event)

    def fixPos(self):
        self.blockSignals(True)
        self.setX(round(self.x()))
        self.setY(round(self.y()))
        self.blockSignals(False)

    def boundingRect(self):
        return QRectF(0, 0, self.p_width, self.p_height)

    def paint(self, painter, option, widget):
        painter.save()
        painter.setRenderHint(QPainter.Antialiasing, bool(options.antialiasing == ANTIALIASING_FULL))
        rect = QRectF(0, 0, self.p_width, self.p_height)

        # Draw rectangle
        pen = QPen(canvas.theme.box_pen_sel if self.isSelected() else canvas.theme.box_pen)
        pen.setWidthF(pen.widthF() + 0.00001)
        painter.setPen(pen)
        lineHinting = pen.widthF() / 2

        if canvas.theme.box_bg_type == Theme.THEME_BG_GRADIENT:
            box_gradient = QLinearGradient(0, 0, 0, self.p_height)
            box_gradient.setColorAt(0, canvas.theme.box_bg_1)
            box_gradient.setColorAt(1, canvas.theme.box_bg_2)
            painter.setBrush(box_gradient)
        else:
            painter.setBrush(canvas.theme.box_bg_1)

        rect.adjust(lineHinting, lineHinting, -lineHinting, -lineHinting)
        painter.drawRect(rect)

        # Draw plugin inline display if supported
        self.paintInlineDisplay(painter)

        # Draw pixmap header
        rect.setHeight(canvas.theme.box_header_height)
        if canvas.theme.box_header_pixmap:
            painter.setPen(Qt.NoPen)
            painter.setBrush(canvas.theme.box_bg_2)

            # outline
            rect.adjust(lineHinting, lineHinting, -lineHinting, -lineHinting)
            painter.drawRect(rect)

            rect.adjust(1, 1, -1, 0)
            painter.drawTiledPixmap(rect, canvas.theme.box_header_pixmap, rect.topLeft())

        # Draw text
        painter.setFont(self.m_font_name)

        if self.isSelected():
            painter.setPen(canvas.theme.box_text_sel)
        else:
            painter.setPen(canvas.theme.box_text)

        if canvas.theme.box_use_icon:
            textPos = QPointF(25, canvas.theme.box_text_ypos)
        else:
            appNameSize = fontHorizontalAdvance(self.m_font_name, self.m_group_name)
            rem = self.p_width - appNameSize
            textPos = QPointF(rem/2, canvas.theme.box_text_ypos)

        painter.drawText(textPos, self.m_group_name)

        self.repaintLines()

        painter.restore()

    def paintInlineDisplay(self, painter):
        if self.m_plugin_inline == self.INLINE_DISPLAY_DISABLED:
            return
        if not options.inline_displays:
            return

        inwidth  = self.p_width - 16 - self.p_width_in - self.p_width_out
        inheight = self.p_height - 3 - canvas.theme.box_header_height - canvas.theme.box_header_spacing - canvas.theme.port_spacing

        scaling  = canvas.scene.getScaleFactor() * canvas.scene.getDevicePixelRatioF()

        if self.m_plugin_id >= 0 and self.m_plugin_id <= MAX_PLUGIN_ID_ALLOWED and (
           self.m_plugin_inline == self.INLINE_DISPLAY_ENABLED or self.m_inline_scaling != scaling):
            if self.m_inline_first:
                size = "%i:%i" % (int(50*scaling), int(50*scaling))
            else:
                size = "%i:%i" % (int(inwidth*scaling), int(inheight*scaling))
            data = canvas.callback(ACTION_INLINE_DISPLAY, self.m_plugin_id, 0, size)
            if data is None:
                return

            self.m_inline_image   = QImage(data['data'], data['width'], data['height'], data['stride'],
                                           QImage.Format_ARGB32)
            self.m_inline_scaling = scaling
            self.m_plugin_inline  = self.INLINE_DISPLAY_CACHED

            # make room for inline display, in a square shape
            if self.m_inline_first:
                self.m_inline_first = False
                aspectRatio   = data['width'] / data['height']
                self.p_height = int(max(50*scaling, self.p_height))
                self.p_width += int(max(0, min((80 - 14)*scaling, (inheight-inwidth) * aspectRatio * scaling)))
                self.repositionPorts()
                self.repaintLines(True)
                self.update()
                return

        if self.m_inline_image is None:
            print("ERROR: inline display image is None for", self.m_plugin_id, self.m_group_name)
            return

        swidth = self.m_inline_image.width() / scaling
        sheight = self.m_inline_image.height() / scaling

        srcx = int(self.p_width_in + (self.p_width - self.p_width_in - self.p_width_out) / 2 - swidth / 2)
        srcy = int(canvas.theme.box_header_height + canvas.theme.box_header_spacing + 1 + (inheight - sheight) / 2)

        painter.drawImage(QRectF(srcx, srcy, swidth, sheight), self.m_inline_image)

# ------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Scalable Button, a custom Qt widget
# Copyright (C) 2013-2020 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ---------------------------------------------------------------------------------------------------------------------
# Imports (Global)

from PyQt5.QtCore import QPointF, QRectF
from PyQt5.QtGui import QColor, QFont, QPainter, QPixmap
from PyQt5.QtSvg import QSvgWidget
from PyQt5.QtWidgets import QPushButton

# ---------------------------------------------------------------------------------------------------------------------
# Widget Class

class ScalableButton(QPushButton):
    def __init__(self, parent):
        QPushButton.__init__(self, parent)

        self.fImageNormal  = None
        self.fImageDown    = None
        self.fImageHover   = None
        self.fImageRect    = QRectF()
        self.fIsHovered    = False
        self.fTopText      = ""
        self.fTopTextColor = QColor()
        self.fTopTextFont  = QFont()
        self.setText("")

    def setPixmaps(self, normal, down, hover):
        self.fImageNormal = QPixmap()
        self.fImageDown   = QPixmap()
        self.fImageHover  = QPixmap()
        self._loadImages(normal, down, hover)

    def setSvgs(self, normal, down, hover):
        self.fImageNormal = QSvgWidget()
        self.fImageDown   = QSvgWidget()
        self.fImageHover  = QSvgWidget()
        self._loadImages(normal, down, hover)

    def _loadImages(self, normal, down, hover):
        self.fImageNormal.load(normal)
        self.fImageDown.load(down)
        self.fImageHover.load(hover)

        if isinstance(self.fImageNormal, QPixmap):
            width  = self.fImageNormal.width()
            height = self.fImageNormal.height()
        else:
            width  = self.fImageNormal.sizeHint().width()
            height = self.fImageNormal.sizeHint().height()

        self.fImageRect = QRectF(0, 0, width, height)

        self.setMinimumSize(width, height)
        self.setMaximumSize(width, height)

    def setTopText(self, text, color, font):
        self.fTopText      = text
        self.fTopTextColor = color
        self.fTopTextFont  = font

    def enterEvent(self, event):
        self.fIsHovered = True
        QPushButton.enterEvent(self, event)

    def leaveEvent(self, event):
        self.fIsHovered = False
        QPushButton.leaveEvent(self, event)

    def paintEvent(self, event):
        painter = QPainter(self)
        event.accept()

        if self.fImageNormal is None:
            return

        if isinstance(self.fImageNormal, QPixmap):
            self.paintEventPixmap(painter)
        else:
            self.paintEventSvg(painter)

        if not self.fTopText:
            return

        painter.save()
        painter.setPen(self.fTopTextColor)
        painter.setBrush(self.fTopTextColor)
        painter.setFont(self.fTopTextFont)
        painter.drawText(QPointF(10, 16), self.fTopText)
        painter.restore()

    def paintEventPixmap(self, painter):
        if not self.isEnabled():
            painter.save()
            painter.setOpacity(0.2)
            painter.drawPixmap(self.fImageRect, self.fImageNormal, self.fImageRect)
            painter.restore()

        elif self.isChecked() or self.isDown():
            painter.drawPixmap(self.fImageRect, self.fImageDown, self.fImageRect)

        elif self.fIsHovered:
            painter.drawPixmap(self.fImageRect, self.fImageHover, self.fImageRect)

        else:
            painter.drawPixmap(self.fImageRect, self.fImageNormal, self.fImageRect)

    def paintEventSvg(self, painter):
        if not self.isEnabled():
            painter.save()
            painter.setOpacity(0.2)
            self.fImageNormal.renderer().render(painter, self.fImageRect)
            painter.restore()

        elif self.isChecked() or self.isDown():
            self.fImageDown.renderer().render(painter, self.fImageRect)

        elif self.fIsHovered:
            self.fImageHover.renderer().render(painter, self.fImageRect)

        else:
            self.fImageNormal.renderer().render(painter, self.fImageRect)

# ---------------------------------------------------------------------------------------------------------------------



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../../resources/ui/carla_edit.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PluginEdit(object):
    def setupUi(self, PluginEdit):
        PluginEdit.setObjectName("PluginEdit")
        PluginEdit.resize(537, 525)
        self.verticalLayout = QtWidgets.QVBoxLayout(PluginEdit)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(PluginEdit)
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.South)
        self.tabWidget.setObjectName("tabWidget")
        self.tabEdit = QtWidgets.QWidget()
        self.tabEdit.setObjectName("tabEdit")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tabEdit)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBox_3 = QtWidgets.QGroupBox(self.tabEdit)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem)
        self.label_ctrl_channel = QtWidgets.QLabel(self.groupBox_3)
        self.label_ctrl_channel.setObjectName("label_ctrl_channel")
        self.horizontalLayout_5.addWidget(self.label_ctrl_channel)
        self.sb_ctrl_channel = QtWidgets.QSpinBox(self.groupBox_3)
        self.sb_ctrl_channel.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.sb_ctrl_channel.setAlignment(QtCore.Qt.AlignCenter)
        self.sb_ctrl_channel.setMinimum(0)
        self.sb_ctrl_channel.setMaximum(16)
        self.sb_ctrl_channel.setProperty("value", 0)
        self.sb_ctrl_channel.setObjectName("sb_ctrl_channel")
        self.horizontalLayout_5.addWidget(self.sb_ctrl_channel)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem1)
        self.verticalLayout_5.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        spacerItem2 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem2)
        self.dial_drywet = ScalableDial(self.groupBox_3)
        self.dial_drywet.setMinimumSize(QtCore.QSize(34, 34))
        self.dial_drywet.setMaximumSize(QtCore.QSize(34, 34))
        self.dial_drywet.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_drywet.setObjectName("dial_drywet")
        self.horizontalLayout_8.addWidget(self.dial_drywet)
        self.dial_vol = ScalableDial(self.groupBox_3)
        self.dial_vol.setMinimumSize(QtCore.QSize(34, 34))
        self.dial_vol.setMaximumSize(QtCore.QSize(34, 34))
        self.dial_vol.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_vol.setObjectName("dial_vol")
        self.horizontalLayout_8.addWidget(self.dial_vol)
        self.stackedWidget = QtWidgets.QStackedWidget(self.groupBox_3)
        self.stackedWidget.setMaximumSize(QtCore.QSize(16777215, 42))
        self.stackedWidget.setLineWidth(0)
        self.stackedWidget.setObjectName("stackedWidget")
        self.page_bal = QtWidgets.QWidget()
        self.page_bal.setObjectName("page_bal")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.page_bal)
        self.horizontalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_6.setSpacing(0)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.dial_b_left = ScalableDial(self.page_bal)
        self.dial_b_left.setMinimumSize(QtCore.QSize(26, 26))
        self.dial_b_left.setMaximumSize(QtCore.QSize(26, 26))
        self.dial_b_left.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_b_left.setObjectName("dial_b_left")
        self.horizontalLayout_6.addWidget(self.dial_b_left)
        self.dial_b_right = ScalableDial(self.page_bal)
        self.dial_b_right.setMinimumSize(QtCore.QSize(26, 26))
        self.dial_b_right.setMaximumSize(QtCore.QSize(26, 26))
        self.dial_b_right.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_b_right.setObjectName("dial_b_right")
        self.horizontalLayout_6.addWidget(self.dial_b_right)
        self.stackedWidget.addWidget(self.page_bal)
        self.page_pan = QtWidgets.QWidget()
        self.page_pan.setObjectName("page_pan")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.page_pan)
        self.horizontalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_7.setSpacing(0)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.dial_pan = ScalableDial(self.page_pan)
        self.dial_pan.setMinimumSize(QtCore.QSize(26, 26))
        self.dial_pan.setMaximumSize(QtCore.QSize(26, 26))
        self.dial_pan.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dial_pan.setObjectName("dial_pan")
        self.horizontalLayout_7.addWidget(self.dial_pan)
        self.stackedWidget.addWidget(self.page_pan)
        self.horizontalLayout_8.addWidget(self.stackedWidget)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.rb_balance = QtWidgets.QRadioButton(self.groupBox_3)
        self.rb_balance.setChecked(True)
        self.rb_balance.setObjectName("rb_balance")
        self.verticalLayout_2.addWidget(self.rb_balance)
        self.rb_pan = QtWidgets.QRadioButton(self.groupBox_3)
        self.rb_pan.setObjectName("rb_pan")
        self.verticalLayout_2.addWidget(self.rb_pan)
        self.horizontalLayout_8.addLayout(self.verticalLayout_2)
        spacerItem3 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_8.addItem(spacerItem3)
        self.verticalLayout_5.addLayout(self.horizontalLayout_8)
        self.gridLayout_2.addWidget(self.groupBox_3, 2, 0, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.tabEdit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.ch_use_chunks = QtWidgets.QCheckBox(self.groupBox)
        self.ch_use_chunks.setObjectName("ch_use_chunks")
        self.verticalLayout_4.addWidget(self.ch_use_chunks)
        self.line_2 = QtWidgets.QFrame(self.groupBox)
        self.line_2.setLineWidth(0)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_4.addWidget(self.line_2)
        self.label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout_4.addWidget(self.label)
        self.ch_fixed_buffer = QtWidgets.QCheckBox(self.groupBox)
        self.ch_fixed_buffer.setObjectName("ch_fixed_buffer")
        self.verticalLayout_4.addWidget(self.ch_fixed_buffer)
        self.ch_force_stereo = QtWidgets.QCheckBox(self.groupBox)
        self.ch_force_stereo.setObjectName("ch_force_stereo")
        self.verticalLayout_4.addWidget(self.ch_force_stereo)
        self.line = QtWidgets.QFrame(self.groupBox)
        self.line.setLineWidth(0)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_4.addWidget(self.line)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_4.addWidget(self.label_2)
        self.ch_map_program_changes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_map_program_changes.setObjectName("ch_map_program_changes")
        self.verticalLayout_4.addWidget(self.ch_map_program_changes)
        self.ch_send_notes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_notes.setObjectName("ch_send_notes")
        self.verticalLayout_4.addWidget(self.ch_send_notes)
        self.ch_send_program_changes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_program_changes.setObjectName("ch_send_program_changes")
        self.verticalLayout_4.addWidget(self.ch_send_program_changes)
        self.ch_send_control_changes = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_control_changes.setObjectName("ch_send_control_changes")
        self.verticalLayout_4.addWidget(self.ch_send_control_changes)
        self.ch_send_channel_pressure = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_channel_pressure.setObjectName("ch_send_channel_pressure")
        self.verticalLayout_4.addWidget(self.ch_send_channel_pressure)
        self.ch_send_note_aftertouch = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_note_aftertouch.setObjectName("ch_send_note_aftertouch")
        self.verticalLayout_4.addWidget(self.ch_send_note_aftertouch)
        self.ch_send_pitchbend = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_pitchbend.setObjectName("ch_send_pitchbend")
        self.verticalLayout_4.addWidget(self.ch_send_pitchbend)
        self.ch_send_all_sound_off = QtWidgets.QCheckBox(self.groupBox)
        self.ch_send_all_sound_off.setObjectName("ch_send_all_sound_off")
        self.verticalLayout_4.addWidget(self.ch_send_all_sound_off)
        self.gridLayout_2.addWidget(self.groupBox, 1, 1, 2, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem4 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.label_plugin = QtWidgets.QLabel(self.tabEdit)
        self.label_plugin.setAlignment(QtCore.Qt.AlignCenter)
        self.label_plugin.setWordWrap(True)
        self.label_plugin.setObjectName("label_plugin")
        self.horizontalLayout.addWidget(self.label_plugin)
        spacerItem5 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem5)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.sw_programs = QtWidgets.QStackedWidget(self.tabEdit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sw_programs.sizePolicy().hasHeightForWidth())
        self.sw_programs.setSizePolicy(sizePolicy)
        self.sw_programs.setLineWidth(0)
        self.sw_programs.setObjectName("sw_programs")
        self.sww_programs = QtWidgets.QWidget()
        self.sww_programs.setObjectName("sww_programs")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.sww_programs)
        self.horizontalLayout_3.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem6 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem6)
        self.label_programs = QtWidgets.QLabel(self.sww_programs)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_programs.sizePolicy().hasHeightForWidth())
        self.label_programs.setSizePolicy(sizePolicy)
        self.label_programs.setObjectName("label_programs")
        self.horizontalLayout_3.addWidget(self.label_programs)
        self.cb_programs = QtWidgets.QComboBox(self.sww_programs)
        self.cb_programs.setMinimumSize(QtCore.QSize(150, 0))
        self.cb_programs.setObjectName("cb_programs")
        self.horizontalLayout_3.addWidget(self.cb_programs)
        self.sw_programs.addWidget(self.sww_programs)
        self.sww_midiPrograms = QtWidgets.QWidget()
        self.sww_midiPrograms.setObjectName("sww_midiPrograms")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.sww_midiPrograms)
        self.horizontalLayout_4.setContentsMargins(2, 2, 2, 2)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        spacerItem7 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem7)
        self.label_midi_programs = QtWidgets.QLabel(self.sww_midiPrograms)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_midi_programs.sizePolicy().hasHeightForWidth())
        self.label_midi_programs.setSizePolicy(sizePolicy)
        self.label_midi_programs.setObjectName("label_midi_programs")
        self.horizontalLayout_4.addWidget(self.label_midi_programs)
        self.cb_midi_programs = QtWidgets.QComboBox(self.sww_midiPrograms)
        self.cb_midi_programs.setMinimumSize(QtCore.QSize(150, 0))
        self.cb_midi_programs.setObjectName("cb_midi_programs")
        self.horizontalLayout_4.addWidget(self.cb_midi_programs)
        self.sw_programs.addWidget(self.sww_midiPrograms)
        self.gridLayout.addWidget(self.sw_programs, 0, 1, 1, 2)
        self.b_save_state = QtWidgets.QPushButton(self.tabEdit)
        self.b_save_state.setFocusPolicy(QtCore.Qt.NoFocus)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/16x16/document-save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_save_state.setIcon(icon)
        self.b_save_state.setObjectName("b_save_state")
        self.gridLayout.addWidget(self.b_save_state, 1, 1, 1, 1)
        self.b_load_state = QtWidgets.QPushButton(self.tabEdit)
        self.b_load_state.setFocusPolicy(QtCore.Qt.NoFocus)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/16x16/document-open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.b_load_state.setIcon(icon1)
        self.b_load_state.setObjectName("b_load_state")
        self.gridLayout.addWidget(self.b_load_state, 1, 2, 1, 1)
        spacerItem8 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem8, 1, 0, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 0, 1, 2)
        self.groupBox_2 = QtWidgets.QGroupBox(self.tabEdit)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.le_maker = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_maker.setInputMask("")
        self.le_maker.setText("")
        self.le_maker.setFrame(False)
        self.le_maker.setReadOnly(True)
        self.le_maker.setObjectName("le_maker")
        self.gridLayout_3.addWidget(self.le_maker, 3, 1, 1, 1)
        self.label_label = QtWidgets.QLabel(self.groupBox_2)
        self.label_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_label.setObjectName("label_label")
        self.gridLayout_3.addWidget(self.label_label, 2, 0, 1, 1)
        self.label_name = QtWidgets.QLabel(self.groupBox_2)
        self.label_name.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_name.setObjectName("label_name")
        self.gridLayout_3.addWidget(self.label_name, 1, 0, 1, 1)
        self.le_name = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_name.setInputMask("")
        self.le_name.setText("")
        self.le_name.setFrame(False)
        self.le_name.setReadOnly(True)
        self.le_name.setObjectName("le_name")
        self.gridLayout_3.addWidget(self.le_name, 1, 1, 1, 1)
        self.le_type = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_type.setInputMask("")
        self.le_type.setText("")
        self.le_type.setFrame(False)
        self.le_type.setReadOnly(True)
        self.le_type.setObjectName("le_type")
        self.gridLayout_3.addWidget(self.le_type, 0, 1, 1, 1)
        self.le_copyright = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_copyright.setInputMask("")
        self.le_copyright.setText("")
        self.le_copyright.setFrame(False)
        self.le_copyright.setReadOnly(True)
        self.le_copyright.setObjectName("le_copyright")
        self.gridLayout_3.addWidget(self.le_copyright, 4, 1, 1, 1)
        self.le_unique_id = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_unique_id.setInputMask("")
        self.le_unique_id.setText("")
        self.le_unique_id.setFrame(False)
        self.le_unique_id.setReadOnly(True)
        self.le_unique_id.setObjectName("le_unique_id")
        self.gridLayout_3.addWidget(self.le_unique_id, 5, 1, 1, 1)
        self.label_type = QtWidgets.QLabel(self.groupBox_2)
        self.label_type.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_type.setObjectName("label_type")
        self.gridLayout_3.addWidget(self.label_type, 0, 0, 1, 1)
        self.label_maker = QtWidgets.QLabel(self.groupBox_2)
        self.label_maker.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_maker.setObjectName("label_maker")
        self.gridLayout_3.addWidget(self.label_maker, 3, 0, 1, 1)
        self.label_copyright = QtWidgets.QLabel(self.groupBox_2)
        self.label_copyright.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_copyright.setObjectName("label_copyright")
        self.gridLayout_3.addWidget(self.label_copyright, 4, 0, 1, 1)
        self.label_unique_id = QtWidgets.QLabel(self.groupBox_2)
        self.label_unique_id.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_unique_id.setObjectName("label_unique_id")
        self.gridLayout_3.addWidget(self.label_unique_id, 5, 0, 1, 1)
        self.le_label = QtWidgets.QLineEdit(self.groupBox_2)
        self.le_label.setInputMask("")
        self.le_label.setText("")
        self.le_label.setFrame(False)
        self.le_label.setReadOnly(True)
        self.le_label.setObjectName("le_label")
        self.gridLayout_3.addWidget(self.le_label, 2, 1, 1, 1)
        self.verticalLayout_3.addLayout(self.gridLayout_3)
        self.gridLayout_2.addWidget(self.groupBox_2, 1, 0, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(1, 1, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.MinimumExpanding)
        self.gridLayout_2.addItem(spacerItem9, 3, 0, 1, 2)
        self.tabWidget.addTab(self.tabEdit, "")
        self.verticalLayout.addWidget(self.tabWidget)

        self.retranslateUi(PluginEdit)
        self.tabWidget.setCurrentIndex(0)
        self.stackedWidget.setCurrentIndex(0)
        self.sw_programs.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(PluginEdit)

    def retranslateUi(self, PluginEdit):
        _translate = QtCore.QCoreApplication.translate
        PluginEdit.setWindowTitle(_translate("PluginEdit", "Plugin Editor"))
        self.groupBox_3.setTitle(_translate("PluginEdit", "Control"))
        self.label_ctrl_channel.setText(_translate("PluginEdit", "MIDI Control Channel:"))
        self.sb_ctrl_channel.setSpecialValueText(_translate("PluginEdit", "N"))
        self.dial_drywet.setStatusTip(_translate("PluginEdit", "Output dry/wet (100%)"))
        self.dial_vol.setStatusTip(_translate("PluginEdit", "Output volume (100%)"))
        self.dial_b_left.setStatusTip(_translate("PluginEdit", "Balance Left (0%)"))
        self.dial_b_right.setStatusTip(_translate("PluginEdit", "Balance Right (0%)"))
        self.dial_pan.setStatusTip(_translate("PluginEdit", "Balance Right (0%)"))
        self.rb_balance.setText(_translate("PluginEdit", "Use Balance"))
        self.rb_pan.setText(_translate("PluginEdit", "Use Panning"))
        self.groupBox.setTitle(_translate("PluginEdit", "Settings"))
        self.ch_use_chunks.setText(_translate("PluginEdit", "Use Chunks"))
        self.label.setText(_translate("PluginEdit", "    Audio:"))
        self.ch_fixed_buffer.setText(_translate("PluginEdit", "Fixed-Size Buffer"))
        self.ch_force_stereo.setText(_translate("PluginEdit", "Force Stereo (needs reload)"))
        self.label_2.setText(_translate("PluginEdit", "    MIDI:"))
        self.ch_map_program_changes.setText(_translate("PluginEdit", "Map Program Changes"))
        self.ch_send_notes.setText(_translate("PluginEdit", "Send Notes"))
        self.ch_send_program_changes.setText(_translate("PluginEdit", "Send Bank/Program Changes"))
        self.ch_send_control_changes.setText(_translate("PluginEdit", "Send Control Changes"))
        self.ch_send_channel_pressure.setText(_translate("PluginEdit", "Send Channel Pressure"))
        self.ch_send_note_aftertouch.setText(_translate("PluginEdit", "Send Note Aftertouch"))
        self.ch_send_pitchbend.setText(_translate("PluginEdit", "Send Pitchbend"))
        self.ch_send_all_sound_off.setText(_translate("PluginEdit", "Send All Sound/Notes Off"))
        self.label_plugin.setText(_translate("PluginEdit", "\n"
"Plugin Name\n"
""))
        self.label_programs.setText(_translate("PluginEdit", "Program:"))
        self.label_midi_programs.setText(_translate("PluginEdit", "MIDI Program:"))
        self.b_save_state.setText(_translate("PluginEdit", "Save State"))
        self.b_load_state.setText(_translate("PluginEdit", "Load State"))
        self.groupBox_2.setTitle(_translate("PluginEdit", "Information"))
        self.label_label.setText(_translate("PluginEdit", "Label/URI:"))
        self.label_name.setText(_translate("PluginEdit", "Name:"))
        self.label_type.setText(_translate("PluginEdit", "Type:"))
        self.label_maker.setText(_translate("PluginEdit", "Maker:"))
        self.label_copyright.setText(_translate("PluginEdit", "Copyright:"))
        self.label_unique_id.setText(_translate("PluginEdit", "Unique ID:"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabEdit), _translate("PluginEdit", "Edit"))
from widgets.scalabledial import ScalableDial
import resources_rc



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# PatchBay Canvas engine using QGraphicsView/Scene
# Copyright (C) 2010-2019 Filipe Coelho <falktx@falktx.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of
# the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# For a full copy of the GNU General Public License see the doc/GPL.txt file.

# ------------------------------------------------------------------------------------------------------------
# Imports (Global)

from PyQt5.QtCore import qWarning, Qt, QLineF
from PyQt5.QtGui import QPainter, QPen
from PyQt5.QtWidgets import QGraphicsLineItem

# ------------------------------------------------------------------------------------------------------------
# Imports (Custom)

from . import (
    canvas,
    options,
    port_mode2str,
    port_type2str,
    CanvasLineMovType,
    PORT_MODE_INPUT,
    PORT_MODE_OUTPUT,
    PORT_TYPE_AUDIO_JACK,
    PORT_TYPE_MIDI_ALSA,
    PORT_TYPE_MIDI_JACK,
    PORT_TYPE_PARAMETER,
)

# ------------------------------------------------------------------------------------------------------------

class CanvasLineMov(QGraphicsLineItem):
    def __init__(self, port_mode, port_type, parent):
        QGraphicsLineItem.__init__(self)
        self.setParentItem(parent)

        self.m_port_mode = port_mode
        self.m_port_type = port_type

        # Port position doesn't change while moving around line
        self.p_lineX = self.scenePos().x()
        self.p_lineY = self.scenePos().y()
        self.p_width = parent.getPortWidth()

        if port_type == PORT_TYPE_AUDIO_JACK:
            pen = QPen(canvas.theme.line_audio_jack, 2)
        elif port_type == PORT_TYPE_MIDI_JACK:
            pen = QPen(canvas.theme.line_midi_jack, 2)
        elif port_type == PORT_TYPE_MIDI_ALSA:
            pen = QPen(canvas.theme.line_midi_alsa, 2)
        elif port_type == PORT_TYPE_PARAMETER:
            pen = QPen(canvas.theme.line_parameter, 2)
        else:
            qWarning("PatchCanvas::CanvasLineMov({}, {}, {}) - invalid port type".format(
                     port_mode2str(port_mode), port_type2str(port_type), parent))
            pen = QPen(Qt.black)

        pen.setCapStyle(Qt.RoundCap)
        pen.setWidthF(pen.widthF() + 0.00001)
        self.setPen(pen)

    def updateLinePos(self, scenePos):
        item_pos = [0, 0]

        if self.m_port_mode == PORT_MODE_INPUT:
            item_pos[0] = 0
            item_pos[1] = float(canvas.theme.port_height)/2
        elif self.m_port_mode == PORT_MODE_OUTPUT:
            item_pos[0] = self.p_width + 12
            item_pos[1] = float(canvas.theme.port_height)/2
        else:
            return

        line = QLineF(item_pos[0], item_pos[1], scenePos.x() - self.p_lineX, scenePos.y() - self.p_lineY)
        self.setLine(line)

    def type(self):
        return CanvasLineMovType

    def paint(self, painter, option, widget):
        painter.save()
        painter.setRenderHint(QPainter.Antialiasing, bool(options.antialiasing))
        QGraphicsLineItem.paint(self, painter, option, widget)
        painter.restore()

# ------------------------------------------------------------------------------------------------------------
